package gov.usgs.earthquake.nshmp.netcdf.reader;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkState;
import static gov.usgs.earthquake.nshmp.netcdf.reader.NetcdfUtils.checkXySequence;

import gov.usgs.earthquake.nshmp.data.XySequence;
import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.netcdf.data.BoundingData;
import gov.usgs.earthquake.nshmp.netcdf.data.StaticData;
import gov.usgs.earthquake.nshmp.netcdf.data.StaticDataHazardCurves;

public class HazardNetcdfUtils {
  /**
   * Check whether bounding hazards contain the same: Site classes, IMTs per
   * each site class, and ground motions per each IMT
   *
   * @param a Bounding hazard map A
   * @param b Bounding hazard map B
   */
  static void checkBoundingHazard(
      StaticData<StaticDataHazardCurves> a,
      StaticData<StaticDataHazardCurves> b) {
    checkState(a.size() == b.size(), "Maps are not the same size");
    checkState(a.keySet().containsAll(b.keySet()), "Site classes do not match");
    a.keySet().forEach(key -> checkHazards(a.get(key), b.get(key)));
  }

  /**
   * Checks bounding hazard maps contain the same: Site classes, IMTs per each
   * site class, and ground motions per each IMT
   *
   * @param boundingData The bounding hazards
   */
  static void checkBoundingHazards(
      BoundingData<StaticDataHazardCurves> boundingData,
      Location location) {
    checkArgument(boundingData.containsKey(location), "Location not in bounding hazards");
    boundingData.keySet().stream()
        .filter(loc -> loc.equals(location))
        .forEach(key -> {
          checkBoundingHazard(boundingData.get(location), boundingData.get(key));
        });
  }

  /**
   * Check whether hazards contain the same: IMTs and ground motions per each
   * IMT
   *
   * @param a Hazard A
   * @param b Hazard B
   */
  static void checkHazards(
      StaticDataHazardCurves a,
      StaticDataHazardCurves b) {
    checkState(a.size() == b.size(), "Maps are not the same size");
    checkState(a.keySet().containsAll(b.keySet()), "IMTs do not match");
    a.keySet().forEach(key -> checkXySequence(a.get(key), b.get(key)));
  }

  static StaticData<StaticDataHazardCurves> cleanData(StaticData<StaticDataHazardCurves> data) {
    var cleanData = StaticData.<StaticDataHazardCurves> builder();

    data.entrySet()
        .forEach(entry -> {
          var cleanCurves = StaticDataHazardCurves.builder();
          entry.getValue().entrySet().stream()
              .forEach(imtEntry -> {
                cleanCurves.put(imtEntry.getKey(),
                    NetcdfUtils.cleanData(imtEntry.getValue(), NetcdfUtils.HAZARD_PRECISION));
              });
          cleanData.put(entry.getKey(), cleanCurves.build());
        });

    return cleanData.build();
  }

  /*
   * Linear interpolation of data values to a target point
   */
  static StaticData<StaticDataHazardCurves> linearInterpolateHazardCurves(
      StaticData<StaticDataHazardCurves> v1,
      StaticData<StaticDataHazardCurves> v2,
      double frac) {
    checkBoundingHazard(v1, v2);

    var targetMap = StaticData.<StaticDataHazardCurves> builder();

    v1.keySet().forEach(siteClass -> {
      var imtHazards = StaticDataHazardCurves.builder();
      var v1StaticHazards = v1.get(siteClass);
      var v2StaticHazards = v2.get(siteClass);

      v1StaticHazards.keySet().forEach(imt -> {
        var v1Haz = v1StaticHazards.get(imt).yValues().toArray();
        var v2Haz = v2StaticHazards.get(imt).yValues().toArray();
        var target = new double[v1Haz.length];

        for (int i = 0; i < v1Haz.length; i++) {
          target[i] = v1Haz[i] * (1 - frac) + v2Haz[i] * frac;
        }

        var xValues = v1StaticHazards.get(imt).xValues().toArray();
        imtHazards.put(imt, XySequence.create(xValues, target));
      });

      targetMap.put(siteClass, imtHazards.build());
    });

    return cleanData(targetMap.build());
  }
}
