package gov.usgs.earthquake.nshmp.netcdf.reader;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;

import gov.usgs.earthquake.nshmp.Maths;
import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.gmm.NehrpSiteClass;
import gov.usgs.earthquake.nshmp.netcdf.data.NetcdfData;
import gov.usgs.earthquake.nshmp.netcdf.data.NetcdfShape;
import gov.usgs.earthquake.nshmp.netcdf.data.ScienceBaseMetadata;
import gov.usgs.earthquake.nshmp.netcdf.reader.NetcdfUtils.Key;

import ucar.ma2.DataType;
import ucar.nc2.Group;

/**
 * Read in NetCDF file.
 *
 * @author U.S. Geological Survey
 */
public abstract class Reader {

  Group targetGroup;
  NetcdfData netcdfData;
  private NetcdfShape netcdfShape;

  static final String FLAG_KEY = "flag_key";
  static final Gson GSON = new Gson();

  public Reader(Group targetGroup) throws IOException {
    this.targetGroup = targetGroup;
    netcdfData = readData(targetGroup);
    netcdfShape = buildNetcdfShape(targetGroup);
  }

  public NetcdfShape netcdfShape() {
    return netcdfShape;
  }

  public NetcdfData netcdfData() {
    return netcdfData;
  }

  abstract NetcdfShape buildNetcdfShape(Group group);

  NetcdfData readData(Group targetGroup) throws IOException {
    var vSiteClass = targetGroup.findVariableLocal(Key.SITE_CLASS);
    var vImts = targetGroup.findVariableLocal(Key.IMT);
    var vVs30 = targetGroup.findVariableLocal(Key.VS30);
    var stringStringMapType = new TypeToken<Map<String, String>>() {}.getType();

    // get list of IMT enums
    Map<String, String> imtFlagKeys = GSON
        .fromJson(vImts.findAttribute(FLAG_KEY)
            .getValues().toString(), stringStringMapType);
    var imts = Arrays.stream((int[]) vImts.read().get1DJavaArray(DataType.INT))
        .boxed()
        .map(imtInt -> imtFlagKeys.get(Integer.toString((imtInt))))
        .map(imt -> Imt.valueOf(imt))
        .collect(Collectors.toUnmodifiableList());

    // get list of SiteClass enums
    Map<String, String> siteClassFlagKeys = GSON
        .fromJson(vSiteClass.findAttribute(FLAG_KEY)
            .getValues().toString(), stringStringMapType);
    List<NehrpSiteClass> siteClasses =
        Arrays.stream((int[]) vSiteClass.read().get1DJavaArray(DataType.INT)).boxed()
            .map(siteClassInt -> siteClassFlagKeys.get(Integer.toString(siteClassInt)))
            .map(siteClass -> NehrpSiteClass.valueOf(siteClass))
            .collect(Collectors.toUnmodifiableList());

    // Get vs30 map
    Map<String, Double> vs30FlagKeys = GSON
        .fromJson(vVs30.findAttribute(FLAG_KEY).getValues().toString(),
            new TypeToken<Map<String, Double>>() {}.getType());
    Map<NehrpSiteClass, Double> vs30Map = vs30FlagKeys.entrySet().stream()
        .collect(
            Collectors.toUnmodifiableMap(
                entry -> NehrpSiteClass.valueOf(entry.getKey()),
                Map.Entry::getValue));

    var latitudes = Arrays.stream(NetcdfUtils.getDoubleArray(targetGroup, Key.LAT))
        .map(lat -> Maths.round(lat, NetcdfUtils.LOCATION_PRECISION))
        .toArray();

    var longitudes = Arrays.stream(NetcdfUtils.getDoubleArray(targetGroup, Key.LON))
        .map(lon -> Maths.round(lon, NetcdfUtils.LOCATION_PRECISION))
        .toArray();

    return NetcdfData.builder()
        .imts(imts)
        .latitudes(latitudes)
        .longitudes(longitudes)
        .scienceBaseMetadata(new ScienceBaseMetadata(targetGroup))
        .siteClasses(siteClasses)
        .vs30Map(vs30Map)
        .build();
  }
}
